==> js-next-0.10.0.tgz: package/src/plugins/candyMachineModule/Client.upload.ts <==
import { ConfirmOptions, PublicKey } from '@solana/web3.js';
import { Signer } from '@/types';
import { CandyMachineToUpdateNotFoundError } from '@/errors';
import { MetaplexFile } from '@/plugins/storageModule';
import { ConfigLine, Creator } from '@metaplex-foundation/mpl-candy-machine';
import {
  JsonMetadata,
  JsonMetadataFile,
  UploadMetadataInput,
} from '@/plugins/nftModule';
import type { CandyMachineClient } from './CandyMachineClient';
import {
  assertCanAdd,
  assertNotFull,
  creatorsToJsonMetadataCreators,
} from './Client.helpers';
import { randomStr } from '../../utils';

// -----------------
// uploadAssetForCandyMachine (single)
// -----------------
export type UploadAssetProperties = {
  creators?: Creator[];
  files?: JsonMetadataFile<MetaplexFile | string>[];
  [key: string]: unknown;
};

export type UploadAssetToCandyMachineParams = UploadMetadataInput & {
  // Accounts
  candyMachineAddress: PublicKey;
  authoritySigner: Signer;

  // Asset
  properties?: UploadAssetProperties;

  // If `true` the successfully uploaded asset is added to the candy machine
  addToCandyMachine?: boolean;

  // Transaction Options.
  confirmOptions?: ConfirmOptions;
};

export async function uploadAssetForCandyMachine(
  this: CandyMachineClient,
  params: UploadAssetToCandyMachineParams
) {
  const candyMachine = await this.findByAddress(params.candyMachineAddress);
  if (candyMachine == null) {
    throw new CandyMachineToUpdateNotFoundError(params.candyMachineAddress);
  }

  assertNotFull(candyMachine, candyMachine.assetsCount);

  // Default NFT creators to equal those of the Candy Machine
  const creators = params.properties?.creators ?? candyMachine.creators;
  const uploadProperties = {
    ...params.properties,
    creators: creatorsToJsonMetadataCreators(creators),
  };
  // TODO(thlorenz): Is this correct?
  const seller_fee_basis_points =
    params.seller_fee_basis_points ?? candyMachine.sellerFeeBasisPoints;

  const {
    candyMachineAddress,
    authoritySigner,
    properties,
    // TODO(thlorenz): prevent same asset from being uploaded twice, remove once
    // API improves to have clearly separated properties
    assets,
    ...uploadInputParams
  } = params;

  const uploadInput: UploadMetadataInput = {
    ...uploadInputParams,
    properties: uploadProperties,
    seller_fee_basis_points,
  };
  const { uri, metadata } = await this.metaplex
    .nfts()
    .uploadMetadata(uploadInput);

  let addAssetsTransactionId;
  if (params.addToCandyMachine) {
    const assetName = selectAssetName(metadata, params);
    const { transactionId } = await this.addAssets({
      candyMachineAddress: params.candyMachineAddress,
      authoritySigner: params.authoritySigner,
      assets: [{ uri, name: assetName }],
      confirmOptions: params.confirmOptions,
    });
    addAssetsTransactionId = transactionId;
  }

  return {
    metadata,
    uri,
    addAssetsTransactionId,
  };
}

// -----------------
// uploadAssetsForCandyMachine (multiple)
// -----------------

// NOTE: the `[key: string]: unknown;` that is part of `UploadMetadataInput` breaks type safety once
// we extend a derived type, thus we don't get intellisense for `CandyMachineAsset` either
export type CandyMachineAsset = Omit<
  UploadAssetToCandyMachineParams,
  'candyMachineAddress' | 'authoritySigner'
>;

export type UploadAssetsToCandyMachineParams = {
  // Accounts
  candyMachineAddress: PublicKey;
  authoritySigner: Signer;

  // Assets
  assets: MetaplexFile[];

  // If `true` then the assets are uploaded in parallel, however note that this
  // can result in some successfully uploading while others fail
  parallel?: boolean;

  // If `true` all successfully uploaded assets are added to the candy machine
  addToCandyMachine?: boolean;

  // Transaction Options.
  confirmOptions?: ConfirmOptions;
};

export type UploadedAsset = {
  uri: string;
  metadata: JsonMetadata<string>;
  name: string;
};

export async function uploadAssetsForCandyMachine(
  this: CandyMachineClient,
  params: UploadAssetsToCandyMachineParams
) {
  const candyMachine = await this.findByAddress(params.candyMachineAddress);
  if (candyMachine == null) {
    throw new CandyMachineToUpdateNotFoundError(params.candyMachineAddress);
  }

  assertNotFull(candyMachine, candyMachine.assetsCount);
  assertCanAdd(candyMachine, candyMachine.assetsCount, params.assets.length);

  const { parallel = false, addToCandyMachine = false } = params;

  // TODO(thlorenz): prevent same asset from being uploaded twice, remove once
  // API improves to have clearly separated properties
  const { assets, ...assetLessParams } = params;
  const uploadParams = assets.map((x) => {
    const param: UploadAssetToCandyMachineParams = {
      ...assetLessParams,
      image: x,
      name: x.displayName,
      // We add them all in one transaction after all assets are uploaded
      addToCandyMachine: false,
    };
    return param;
  });
  let uploadedAssets: UploadedAsset[] = [];
  const errors = [];

  if (parallel) {
    // NOTE: we are uploading in parallel here but if only one upload was to fail
    // all the other ones still happen as we cannot cancel promises
    const promises = uploadParams.map(async (assetParam) => {
      let uploaded;
      let err;
      try {
        uploaded = await _uploadAssetAndSelectName(this, params, assetParam);
      } catch (e) {
        errors.push(e);
      }

      return { uploaded, err };
    });

    const results = await Promise.all(promises);
    for (const { err, uploaded } of results) {
      if (err) {
        errors.push(err);
      } else {
        uploadedAssets.push(uploaded as UploadedAsset);
      }
    }
  } else {
    for (const assetParam of uploadParams) {
      try {
        uploadedAssets.push(
          await _uploadAssetAndSelectName(this, params, assetParam)
        );
      } catch (err) {
        errors.push(err);
        continue;
      }
    }
  }

  let addAssetsTransactionId;
  let updatedCandyMachine = candyMachine;
  if (addToCandyMachine && uploadedAssets.length > 0) {
    const configLines: ConfigLine[] = uploadedAssets.map((x) => ({
      uri: x.uri,
      name: x.name,
    }));

    const { transactionId, candyMachine } = await this.addAssets({
      ...params,
      assets: configLines,
    });
    addAssetsTransactionId = transactionId;
    if (candyMachine != null) {
      updatedCandyMachine = candyMachine;
    }
  }

  return {
    addAssetsTransactionId,
    uploadedAssets,
    candyMachine: updatedCandyMachine,
    errors,
  };
}

async function _uploadAssetAndSelectName(
  candyMachine: CandyMachineClient,
  params: UploadAssetsToCandyMachineParams,
  asset: CandyMachineAsset
) {
  const { uri, metadata } = await candyMachine.uploadAssetForCandyMachine({
    ...params,
    ...asset,
  });
  return {
    uri,
    metadata,
    name: selectAssetName(metadata, asset),
  };
}

function selectAssetName(
  metadata: JsonMetadata<string>,
  asset: CandyMachineAsset
): string {
  return (
    metadata.name ?? (typeof asset.name === 'string' ? asset.name : randomStr())
  );
}

==> js-next-0.11.1.tgz: package/src/plugins/candyMachineModule/Client.upload.ts <==
import { ConfirmOptions, PublicKey } from '@solana/web3.js';
import { Signer } from '@/types';
import { CandyMachineToUpdateNotFoundError } from '@/errors';
import { MetaplexFile } from '@/plugins/storageModule';
import { ConfigLine } from '@metaplex-foundation/mpl-candy-machine';
import { JsonMetadata, UploadMetadataInput } from '@/plugins/nftModule';
import type { CandyMachineClient } from './CandyMachineClient';
import {
  assertCanAdd,
  assertNotFull,
  creatorsToJsonMetadataCreators,
} from './Client.helpers';
import { randomStr } from '../../utils';

export type UploadAssetToCandyMachineParams = {
  // Accounts.
  candyMachineAddress: PublicKey;
  authoritySigner: Signer;

  // Data.
  metadata: UploadMetadataInput;

  // If `true` the successfully uploaded asset is added to the candy machine.
  addToCandyMachine?: boolean;

  // Transaction Options.
  confirmOptions?: ConfirmOptions;
};

export async function uploadAssetForCandyMachine(
  this: CandyMachineClient,
  params: UploadAssetToCandyMachineParams
) {
  const {
    candyMachineAddress,
    authoritySigner,
    metadata: rawMetadata,
    addToCandyMachine = false,
    confirmOptions,
  } = params;

  const candyMachine = await this.findByAddress(candyMachineAddress);
  if (candyMachine == null) {
    throw new CandyMachineToUpdateNotFoundError(candyMachineAddress);
  }

  assertNotFull(candyMachine, candyMachine.assetsCount);

  const { uri, metadata } = await this.metaplex.nfts().uploadMetadata({
    ...rawMetadata,
    // TODO(thlorenz): Is this correct?
    seller_fee_basis_points:
      rawMetadata.seller_fee_basis_points ?? candyMachine.sellerFeeBasisPoints,
    properties: {
      ...rawMetadata.properties,
      // Default NFT creators to equal those of the Candy Machine
      creators:
        rawMetadata.properties?.creators ??
        creatorsToJsonMetadataCreators(candyMachine.creators),
    },
  });

  let addAssetsTransactionId;
  if (addToCandyMachine) {
    const { transactionId } = await this.addAssets({
      candyMachineAddress: candyMachineAddress,
      authoritySigner: authoritySigner,
      assets: [{ uri, name: metadata.name ?? randomStr() }],
      confirmOptions,
    });
    addAssetsTransactionId = transactionId;
  }

  return {
    metadata,
    uri,
    addAssetsTransactionId,
  };
}

export type UploadAssetsToCandyMachineParams = {
  // Accounts
  candyMachineAddress: PublicKey;
  authoritySigner: Signer;

  // Assets
  assets: MetaplexFile[];

  // If `true` then the assets are uploaded in parallel, however note that this
  // can result in some successfully uploading while others fail
  parallel?: boolean;

  // If `true` all successfully uploaded assets are added to the candy machine
  addToCandyMachine?: boolean;

  // Transaction Options.
  confirmOptions?: ConfirmOptions;
};

export type UploadedAsset = {
  uri: string;
  metadata: JsonMetadata<string>;
  name: string;
};

export async function uploadAssetsForCandyMachine(
  this: CandyMachineClient,
  params: UploadAssetsToCandyMachineParams
) {
  const {
    candyMachineAddress,
    assets,
    parallel = false,
    addToCandyMachine = false,
  } = params;

  const candyMachine = await this.findByAddress(candyMachineAddress);
  if (candyMachine == null) {
    throw new CandyMachineToUpdateNotFoundError(candyMachineAddress);
  }

  assertNotFull(candyMachine, candyMachine.assetsCount);
  assertCanAdd(candyMachine, candyMachine.assetsCount, assets.length);

  // TODO(thlorenz): prevent same asset from being uploaded twice, remove once
  // API improves to have clearly separated properties
  const uploadParams = assets.map(
    (file: MetaplexFile): UploadAssetToCandyMachineParams => ({
      ...params,
      metadata: {
        image: file,
        name: file.displayName,
      },
      // We add them all in one transaction after all assets are uploaded
      addToCandyMachine: false,
    })
  );

  let uploadedAssets: UploadedAsset[] = [];
  const errors = [];

  if (parallel) {
    // NOTE: we are uploading in parallel here but if only one upload was to fail
    // all the other ones still happen as we cannot cancel promises
    const promises = uploadParams.map(async (assetParam) => {
      let uploaded;
      let err;
      try {
        uploaded = await _uploadAssetAndSelectName(this, assetParam);
      } catch (e) {
        errors.push(e);
      }

      return { uploaded, err };
    });

    const results = await Promise.all(promises);
    for (const { err, uploaded } of results) {
      if (err) {
        errors.push(err);
      } else {
        uploadedAssets.push(uploaded as UploadedAsset);
      }
    }
  } else {
    for (const assetParam of uploadParams) {
      try {
        uploadedAssets.push(await _uploadAssetAndSelectName(this, assetParam));
      } catch (err) {
        errors.push(err);
        continue;
      }
    }
  }

  let addAssetsTransactionId;
  let updatedCandyMachine = candyMachine;
  if (addToCandyMachine && uploadedAssets.length > 0) {
    const configLines: ConfigLine[] = uploadedAssets.map((x) => ({
      uri: x.uri,
      name: x.name,
    }));

    const { transactionId, candyMachine } = await this.addAssets({
      ...params,
      assets: configLines,
    });
    addAssetsTransactionId = transactionId;
    if (candyMachine != null) {
      updatedCandyMachine = candyMachine;
    }
  }

  return {
    addAssetsTransactionId,
    uploadedAssets,
    candyMachine: updatedCandyMachine,
    errors,
  };
}

async function _uploadAssetAndSelectName(
  candyMachine: CandyMachineClient,
  params: UploadAssetToCandyMachineParams
): Promise<UploadedAsset> {
  const { uri, metadata: parseMetadata } =
    await candyMachine.uploadAssetForCandyMachine(params);

  return {
    uri,
    metadata: parseMetadata,
    name: parseMetadata.name ?? randomStr(),
  };
}
